/*
classSound.cpp

By: Sean Heber (bigzaphod@legions.com)
Come and visit a cool site:  http://www.legions.com/

You are free to use this class for anything you want.  All that I ask is that
you please credit me and/or e-mail me and let me know what you're doing with it.

Also, if you are distributing source with your application, please keep this
comment and add a note with any changes you may have made to the class and how
it works.

As is standard, I take no responsibility if this causes any damage at all.  No matter what.  Forever.

(Note that this is not being released under the GPL or anything like that, so feel
 free to use it in a commercial program if you want.  I'll be honored if you do :-)

Have fun, drink lots of Dew, and never forget THE answer: 42.
*/


#include "classSound.h"

// The constructor actually loads and stores the sound file into
// memory.
classSound::classSound( char *path, char *name )
{
	OK = false;
	AllowMult = false;  // By default don't allow multiple

	sound = NULL;

	FilePath = new char[ strlen( path ) ];
	strcpy( FilePath, path );

	entry.SetTo( path, true );

	if( entry.Exists( ) && entry.InitCheck( ) == B_OK && LoadSound( path ) )  // Make sure everything is good before I go any farther
	{
		OK = true;
		player = new BSoundPlayer( &fmt, name );  // Make a new player object with the sound type found above and the file that was loaded
		player->SetVolume(1.0);  // Just setting the volume to 1.0 is probably fine.  It just means the sound plays as loud as it can according to where the user set the audio prefs.
	}
}

// Actually load the sound into an object
bool classSound::LoadSound( char *path )
{
	bool Return = false;

	if (entry.GetRef(&ref) == B_OK)
	{
		sound = new BSound(&ref, true);  // Actually load the sound file into memory

		fmt = sound->Format();  // Have Be figure out what the sound file is and set a type up.
			
		if (sound->InitCheck() == B_OK)
			Return = true;
	}
	return Return;
}

// Destroy the player and things (only if it was ever made)
classSound::~classSound( )
{
	if( OK )
	{ 
		sound -> ReleaseRef();
		player -> Stop();
		delete player;
	}
}

// Just plays the sound.  It also checks to make sure that it CAN and is ALLOWED to play the sound
void classSound::PlaySound( ) { 
	if( !OK || ( IsPlaying( ) && !AllowMult ) )
		return;

	if( IsPlaying() )  // If the sound is playing, we need to reload the sound.  This could be a bug.  Or I'm not doing something right.  But it works.
	{
		sound -> ReleaseRef();  // Release the sound
		LoadSound( FilePath );  // Reload the sound file
	}

	if( !IsPlaying() )  // If nothing is playing, we need to reset the BSoundPlayer.  Could be a bug?  It did strange things without this.
	{
		player -> Stop( );  // Stop the player to reset it.  (I don't know if this is a BSoundPlayer bug or something
		player -> Start();  // Init the player object and get it all ready to go.
	}

	id = player->StartPlaying( sound );   // Actually allow Be to start playing (does NOT block!)
}

// Just check to see if the sound it playing
bool classSound::IsPlaying( )
{
	return ( OK && ( player->HasData( ) || player->IsPlaying( id ) ) );
}

// Set the multiple flag.
void classSound::AllowMultiple( bool allow )
{
	AllowMult = allow;
}

// Just checks to make sure all is good.
bool classSound::IsOK( )
{
	return OK;
}